@@ -7,15 +7,26 @@
    https://docs.djangoproject.com/en/dev/ref/contrib/gis/layermapping/
 """
 import sys
-from decimal import Decimal, InvalidOperation as DecimalInvalidOperation
+from decimal import Decimal
+from decimal import InvalidOperation as DecimalInvalidOperation
+from pathlib import Path
 
 from django.contrib.gis.db.models import GeometryField
 from django.contrib.gis.gdal import (
-    CoordTransform, DataSource, GDALException, OGRGeometry, OGRGeomType,
+    CoordTransform,
+    DataSource,
+    GDALException,
+    OGRGeometry,
+    OGRGeomType,
     SpatialReference,
 )
 from django.contrib.gis.gdal.field import (
-    OFTDate, OFTDateTime, OFTInteger, OFTInteger64, OFTReal, OFTString,
+    OFTDate,
+    OFTDateTime,
+    OFTInteger,
+    OFTInteger64,
+    OFTReal,
+    OFTString,
     OFTTime,
 )
 from django.core.exceptions import FieldDoesNotExist, ObjectDoesNotExist
@@ -49,12 +60,12 @@ class LayerMapping:
 
     # Acceptable 'base' types for a multi-geometry type.
     MULTI_TYPES = {
-        1: OGRGeomType('MultiPoint'),
-        2: OGRGeomType('MultiLineString'),
-        3: OGRGeomType('MultiPolygon'),
-        OGRGeomType('Point25D').num: OGRGeomType('MultiPoint25D'),
-        OGRGeomType('LineString25D').num: OGRGeomType('MultiLineString25D'),
-        OGRGeomType('Polygon25D').num: OGRGeomType('MultiPolygon25D'),
+        1: OGRGeomType("MultiPoint"),
+        2: OGRGeomType("MultiLineString"),
+        3: OGRGeomType("MultiPolygon"),
+        OGRGeomType("Point25D").num: OGRGeomType("MultiPoint25D"),
+        OGRGeomType("LineString25D").num: OGRGeomType("MultiLineString25D"),
+        OGRGeomType("Polygon25D").num: OGRGeomType("MultiPolygon25D"),
     }
     # Acceptable Django field types and corresponding acceptable OGR
     # counterparts.
@@ -82,10 +93,19 @@ class LayerMapping:
         models.PositiveSmallIntegerField: (OFTInteger, OFTReal, OFTString),
     }
 
-    def __init__(self, model, data, mapping, layer=0,
-                 source_srs=None, encoding='utf-8',
-                 transaction_mode='commit_on_success',
-                 transform=True, unique=None, using=None):
+    def __init__(
+        self,
+        model,
+        data,
+        mapping,
+        layer=0,
+        source_srs=None,
+        encoding="utf-8",
+        transaction_mode="commit_on_success",
+        transform=True,
+        unique=None,
+        using=None,
+    ):
         """
         A LayerMapping object is initialized using the given Model (not an instance),
         a DataSource (or string path to an OGR-supported data file), and a mapping
@@ -93,7 +113,7 @@ class LayerMapping:
         argument usage.
         """
         # Getting the DataSource and the associated Layer.
-        if isinstance(data, str):
+        if isinstance(data, (str, Path)):
             self.ds = DataSource(data, encoding=encoding)
         else:
             self.ds = data
@@ -131,6 +151,7 @@ class LayerMapping:
             # Making sure the encoding exists, if not a LookupError
             # exception will be thrown.
             from codecs import lookup
+
             lookup(encoding)
             self.encoding = encoding
         else:
@@ -138,7 +159,7 @@ class LayerMapping:
 
         if unique:
             self.check_unique(unique)
-            transaction_mode = 'autocommit'  # Has to be set to autocommit.
+            transaction_mode = "autocommit"  # Has to be set to autocommit.
             self.unique = unique
         else:
             self.unique = None
@@ -146,12 +167,12 @@ class LayerMapping:
         # Setting the transaction decorator with the function in the
         # transaction modes dictionary.
         self.transaction_mode = transaction_mode
-        if transaction_mode == 'autocommit':
+        if transaction_mode == "autocommit":
             self.transaction_decorator = None
-        elif transaction_mode == 'commit_on_success':
+        elif transaction_mode == "commit_on_success":
             self.transaction_decorator = transaction.atomic
         else:
-            raise LayerMapError('Unrecognized transaction mode: %s' % transaction_mode)
+            raise LayerMapError("Unrecognized transaction mode: %s" % transaction_mode)
 
     # #### Checking routines used during initialization ####
     def check_fid_range(self, fid_range):
@@ -188,7 +209,9 @@ class LayerMapping:
             try:
                 idx = ogr_fields.index(ogr_map_fld)
             except ValueError:
-                raise LayerMapError('Given mapping OGR field "%s" not found in OGR Layer.' % ogr_map_fld)
+                raise LayerMapError(
+                    'Given mapping OGR field "%s" not found in OGR Layer.' % ogr_map_fld
+                )
             return idx
 
         # No need to increment through each feature in the model, simply check
@@ -199,32 +222,44 @@ class LayerMapping:
             try:
                 model_field = self.model._meta.get_field(field_name)
             except FieldDoesNotExist:
-                raise LayerMapError('Given mapping field "%s" not in given Model fields.' % field_name)
+                raise LayerMapError(
+                    'Given mapping field "%s" not in given Model fields.' % field_name
+                )
 
             # Getting the string name for the Django field class (e.g., 'PointField').
             fld_name = model_field.__class__.__name__
 
             if isinstance(model_field, GeometryField):
                 if self.geom_field:
-                    raise LayerMapError('LayerMapping does not support more than one GeometryField per model.')
+                    raise LayerMapError(
+                        "LayerMapping does not support more than one GeometryField per "
+                        "model."
+                    )
 
                 # Getting the coordinate dimension of the geometry field.
                 coord_dim = model_field.dim
 
                 try:
                     if coord_dim == 3:
-                        gtype = OGRGeomType(ogr_name + '25D')
+                        gtype = OGRGeomType(ogr_name + "25D")
                     else:
                         gtype = OGRGeomType(ogr_name)
                 except GDALException:
-                    raise LayerMapError('Invalid mapping for GeometryField "%s".' % field_name)
+                    raise LayerMapError(
+                        'Invalid mapping for GeometryField "%s".' % field_name
+                    )
 
                 # Making sure that the OGR Layer's Geometry is compatible.
                 ltype = self.layer.geom_type
-                if not (ltype.name.startswith(gtype.name) or self.make_multi(ltype, model_field)):
-                    raise LayerMapError('Invalid mapping geometry; model has %s%s, '
-                                        'layer geometry type is %s.' %
-                                        (fld_name, '(dim=3)' if coord_dim == 3 else '', ltype))
+                if not (
+                    ltype.name.startswith(gtype.name)
+                    or self.make_multi(ltype, model_field)
+                ):
+                    raise LayerMapError(
+                        "Invalid mapping geometry; model has %s%s, "
+                        "layer geometry type is %s."
+                        % (fld_name, "(dim=3)" if coord_dim == 3 else "", ltype)
+                    )
 
                 # Setting the `geom_field` attribute w/the name of the model field
                 # that is a Geometry.  Also setting the coordinate dimension
@@ -241,15 +276,19 @@ class LayerMapping:
                         try:
                             rel_model._meta.get_field(rel_name)
                         except FieldDoesNotExist:
-                            raise LayerMapError('ForeignKey mapping field "%s" not in %s fields.' %
-                                                (rel_name, rel_model.__class__.__name__))
+                            raise LayerMapError(
+                                'ForeignKey mapping field "%s" not in %s fields.'
+                                % (rel_name, rel_model.__class__.__name__)
+                            )
                     fields_val = rel_model
                 else:
-                    raise TypeError('ForeignKey mapping must be of dictionary type.')
+                    raise TypeError("ForeignKey mapping must be of dictionary type.")
             else:
                 # Is the model field type supported by LayerMapping?
                 if model_field.__class__ not in self.FIELD_TYPES:
-                    raise LayerMapError('Django field type "%s" has no OGR mapping (yet).' % fld_name)
+                    raise LayerMapError(
+                        'Django field type "%s" has no OGR mapping (yet).' % fld_name
+                    )
 
                 # Is the OGR field in the Layer?
                 idx = check_ogr_fld(ogr_name)
@@ -257,8 +296,10 @@ class LayerMapping:
 
                 # Can the OGR field type be mapped to the Django field type?
                 if not issubclass(ogr_field, self.FIELD_TYPES[model_field.__class__]):
-                    raise LayerMapError('OGR field "%s" (of type %s) cannot be mapped to Django %s.' %
-                                        (ogr_field, ogr_field.__name__, fld_name))
+                    raise LayerMapError(
+                        'OGR field "%s" (of type %s) cannot be mapped to Django %s.'
+                        % (ogr_field, ogr_field.__name__, fld_name)
+                    )
                 fields_val = model_field
 
             self.fields[field_name] = fields_val
@@ -277,7 +318,7 @@ class LayerMapping:
             sr = self.layer.srs
 
         if not sr:
-            raise LayerMapError('No source reference system defined.')
+            raise LayerMapError("No source reference system defined.")
         else:
             return sr
 
@@ -293,7 +334,9 @@ class LayerMapping:
             if unique not in self.mapping:
                 raise ValueError
         else:
-            raise TypeError('Unique keyword argument must be set with a tuple, list, or string.')
+            raise TypeError(
+                "Unique keyword argument must be set with a tuple, list, or string."
+            )
 
     # Keyword argument retrieval routines ####
     def feature_kwargs(self, feat):
@@ -314,7 +357,7 @@ class LayerMapping:
                 try:
                     val = self.verify_geom(feat.geom, model_field)
                 except GDALException:
-                    raise LayerMapError('Could not retrieve geometry from feature.')
+                    raise LayerMapError("Could not retrieve geometry from feature.")
             elif isinstance(model_field, models.base.ModelBase):
                 # The related _model_, not a field was passed in -- indicating
                 # another mapping for the related Model.
@@ -346,23 +389,34 @@ class LayerMapping:
         Verify if the OGR Field contents are acceptable to the model field. If
         they are, return the verified value, otherwise raise an exception.
         """
-        if (isinstance(ogr_field, OFTString) and
-                isinstance(model_field, (models.CharField, models.TextField))):
+        if isinstance(ogr_field, OFTString) and isinstance(
+            model_field, (models.CharField, models.TextField)
+        ):
             if self.encoding and ogr_field.value is not None:
                 # The encoding for OGR data sources may be specified here
                 # (e.g., 'cp437' for Census Bureau boundary files).
                 val = force_str(ogr_field.value, self.encoding)
             else:
                 val = ogr_field.value
-            if model_field.max_length and val is not None and len(val) > model_field.max_length:
-                raise InvalidString('%s model field maximum string length is %s, given %s characters.' %
-                                    (model_field.name, model_field.max_length, len(val)))
-        elif isinstance(ogr_field, OFTReal) and isinstance(model_field, models.DecimalField):
+            if (
+                model_field.max_length
+                and val is not None
+                and len(val) > model_field.max_length
+            ):
+                raise InvalidString(
+                    "%s model field maximum string length is %s, given %s characters."
+                    % (model_field.name, model_field.max_length, len(val))
+                )
+        elif isinstance(ogr_field, OFTReal) and isinstance(
+            model_field, models.DecimalField
+        ):
             try:
                 # Creating an instance of the Decimal value to use.
                 d = Decimal(str(ogr_field.value))
             except DecimalInvalidOperation:
-                raise InvalidDecimal('Could not construct decimal from: %s' % ogr_field.value)
+                raise InvalidDecimal(
+                    "Could not construct decimal from: %s" % ogr_field.value
+                )
 
             # Getting the decimal value as a tuple.
             dtup = d.as_tuple()
@@ -383,17 +437,21 @@ class LayerMapping:
             # InvalidDecimal exception.
             if n_prec > max_prec:
                 raise InvalidDecimal(
-                    'A DecimalField with max_digits %d, decimal_places %d must '
-                    'round to an absolute value less than 10^%d.' %
-                    (model_field.max_digits, model_field.decimal_places, max_prec)
+                    "A DecimalField with max_digits %d, decimal_places %d must "
+                    "round to an absolute value less than 10^%d."
+                    % (model_field.max_digits, model_field.decimal_places, max_prec)
                 )
             val = d
-        elif isinstance(ogr_field, (OFTReal, OFTString)) and isinstance(model_field, models.IntegerField):
+        elif isinstance(ogr_field, (OFTReal, OFTString)) and isinstance(
+            model_field, models.IntegerField
+        ):
             # Attempt to convert any OFTReal and OFTString value to an OFTInteger.
             try:
                 val = int(ogr_field.value)
             except ValueError:
-                raise InvalidInteger('Could not construct integer from: %s' % ogr_field.value)
+                raise InvalidInteger(
+                    "Could not construct integer from: %s" % ogr_field.value
+                )
         else:
             val = ogr_field.value
         return val
@@ -410,15 +468,17 @@ class LayerMapping:
         # Constructing and verifying the related model keyword arguments.
         fk_kwargs = {}
         for field_name, ogr_name in rel_mapping.items():
-            fk_kwargs[field_name] = self.verify_ogr_field(feat[ogr_name], rel_model._meta.get_field(field_name))
+            fk_kwargs[field_name] = self.verify_ogr_field(
+                feat[ogr_name], rel_model._meta.get_field(field_name)
+            )
 
         # Attempting to retrieve and return the related model.
         try:
             return rel_model.objects.using(self.using).get(**fk_kwargs)
         except ObjectDoesNotExist:
             raise MissingForeignKey(
-                'No ForeignKey %s model found with keyword arguments: %s' %
-                (rel_model.__name__, fk_kwargs)
+                "No ForeignKey %s model found with keyword arguments: %s"
+                % (rel_model.__name__, fk_kwargs)
             )
 
     def verify_geom(self, geom, model_field):
@@ -454,13 +514,17 @@ class LayerMapping:
         SpatialRefSys = self.spatial_backend.spatial_ref_sys()
         try:
             # Getting the target spatial reference system
-            target_srs = SpatialRefSys.objects.using(self.using).get(srid=self.geo_field.srid).srs
+            target_srs = (
+                SpatialRefSys.objects.using(self.using)
+                .get(srid=self.geo_field.srid)
+                .srs
+            )
 
             # Creating the CoordTransform object
             return CoordTransform(self.source_srs, target_srs)
         except Exception as exc:
             raise LayerMapError(
-                'Could not translate between the data source and model geometry.'
+                "Could not translate between the data source and model geometry."
             ) from exc
 
     def geometry_field(self):
@@ -475,11 +539,21 @@ class LayerMapping:
         Given the OGRGeomType for a geometry and its associated GeometryField,
         determine whether the geometry should be turned into a GeometryCollection.
         """
-        return (geom_type.num in self.MULTI_TYPES and
-                model_field.__class__.__name__ == 'Multi%s' % geom_type.django)
+        return (
+            geom_type.num in self.MULTI_TYPES
+            and model_field.__class__.__name__ == "Multi%s" % geom_type.django
+        )
 
-    def save(self, verbose=False, fid_range=False, step=False,
-             progress=False, silent=False, stream=sys.stdout, strict=False):
+    def save(
+        self,
+        verbose=False,
+        fid_range=False,
+        step=False,
+        progress=False,
+        silent=False,
+        stream=sys.stdout,
+        strict=False,
+    ):
         """
         Save the contents from the OGR DataSource Layer into the database
         according to the mapping dictionary given at initialization.
@@ -545,7 +619,9 @@ class LayerMapping:
                     if strict:
                         raise
                     elif not silent:
-                        stream.write('Ignoring Feature ID %s because: %s\n' % (feat.fid, msg))
+                        stream.write(
+                            "Ignoring Feature ID %s because: %s\n" % (feat.fid, msg)
+                        )
                 else:
                     # Constructing the model using the keyword args
                     is_update = False
@@ -583,23 +659,29 @@ class LayerMapping:
                         m.save(using=self.using)
                         num_saved += 1
                         if verbose:
-                            stream.write('%s: %s\n' % ('Updated' if is_update else 'Saved', m))
+                            stream.write(
+                                "%s: %s\n" % ("Updated" if is_update else "Saved", m)
+                            )
                     except Exception as msg:
                         if strict:
                             # Bailing out if the `strict` keyword is set.
                             if not silent:
                                 stream.write(
-                                    'Failed to save the feature (id: %s) into the '
-                                    'model with the keyword arguments:\n' % feat.fid
+                                    "Failed to save the feature (id: %s) into the "
+                                    "model with the keyword arguments:\n" % feat.fid
                                 )
-                                stream.write('%s\n' % kwargs)
+                                stream.write("%s\n" % kwargs)
                             raise
                         elif not silent:
-                            stream.write('Failed to save %s:\n %s\nContinuing\n' % (kwargs, msg))
+                            stream.write(
+                                "Failed to save %s:\n %s\nContinuing\n" % (kwargs, msg)
+                            )
 
                 # Printing progress information, if requested.
                 if progress and num_feat % progress_interval == 0:
-                    stream.write('Processed %d features, saved %d ...\n' % (num_feat, num_saved))
+                    stream.write(
+                        "Processed %d features, saved %d ...\n" % (num_feat, num_saved)
+                    )
 
             # Only used for status output purposes -- incremental saving uses the
             # values returned here.
@@ -612,7 +694,10 @@ class LayerMapping:
         if step and isinstance(step, int) and step < nfeat:
             # Incremental saving is requested at the given interval (step)
             if default_range:
-                raise LayerMapError('The `step` keyword may not be used in conjunction with the `fid_range` keyword.')
+                raise LayerMapError(
+                    "The `step` keyword may not be used in conjunction with the "
+                    "`fid_range` keyword."
+                )
             beg, num_feat, num_saved = (0, 0, 0)
             indices = range(step, nfeat, step)
             n_i = len(indices)
@@ -629,7 +714,9 @@ class LayerMapping:
                     num_feat, num_saved = _save(step_slice, num_feat, num_saved)
                     beg = end
                 except Exception:  # Deliberately catch everything
-                    stream.write('%s\nFailed to save slice: %s\n' % ('=-' * 20, step_slice))
+                    stream.write(
+                        "%s\nFailed to save slice: %s\n" % ("=-" * 20, step_slice)
+                    )
                     raise
         else:
             # Otherwise, just calling the previously defined _save() function.
