@@ -2,8 +2,10 @@
 Base classes for writing management commands (named commands which can
 be executed through ``django-admin`` or ``manage.py``).
 """
+import argparse
 import os
 import sys
+import warnings
 from argparse import ArgumentParser, HelpFormatter
 from io import TextIOBase
 
@@ -12,6 +14,9 @@ from django.core import checks
 from django.core.exceptions import ImproperlyConfigured
 from django.core.management.color import color_style, no_style
 from django.db import DEFAULT_DB_ALIAS, connections
+from django.utils.deprecation import RemovedInDjango41Warning
+
+ALL_CHECKS = "__all__"
 
 
 class CommandError(Exception):
@@ -26,13 +31,17 @@ class CommandError(Exception):
     error) is the preferred way to indicate that something has gone
     wrong in the execution of a command.
     """
-    pass
+
+    def __init__(self, *args, returncode=1, **kwargs):
+        self.returncode = returncode
+        super().__init__(*args, **kwargs)
 
 
 class SystemCheckError(CommandError):
     """
     The system check framework detected unrecoverable errors.
     """
+
     pass
 
 
@@ -42,15 +51,19 @@ class CommandParser(ArgumentParser):
     SystemExit in several occasions, as SystemExit is unacceptable when a
     command is called programmatically.
     """
-    def __init__(self, *, missing_args_message=None, called_from_command_line=None, **kwargs):
+
+    def __init__(
+        self, *, missing_args_message=None, called_from_command_line=None, **kwargs
+    ):
         self.missing_args_message = missing_args_message
         self.called_from_command_line = called_from_command_line
         super().__init__(**kwargs)
 
     def parse_args(self, args=None, namespace=None):
         # Catch missing argument for a better error message
-        if (self.missing_args_message and
-                not (args or any(not arg.startswith('-') for arg in args))):
+        if self.missing_args_message and not (
+            args or any(not arg.startswith("-") for arg in args)
+        ):
             self.error(self.missing_args_message)
         return super().parse_args(args, namespace)
 
@@ -68,15 +81,17 @@ def handle_default_options(options):
     user commands.
     """
     if options.settings:
-        os.environ['DJANGO_SETTINGS_MODULE'] = options.settings
+        os.environ["DJANGO_SETTINGS_MODULE"] = options.settings
     if options.pythonpath:
         sys.path.insert(0, options.pythonpath)
 
 
 def no_translations(handle_func):
     """Decorator that forces a command to run with translations deactivated."""
+
     def wrapped(*args, **kwargs):
         from django.utils import translation
+
         saved_locale = translation.get_language()
         translation.deactivate_all()
         try:
@@ -85,6 +100,7 @@ def no_translations(handle_func):
             if saved_locale is not None:
                 translation.activate(saved_locale)
         return res
+
     return wrapped
 
 
@@ -93,15 +109,21 @@ class DjangoHelpFormatter(HelpFormatter):
     Customized formatter so that command-specific arguments appear in the
     --help output before arguments common to all commands.
     """
+
     show_last = {
-        '--version', '--verbosity', '--traceback', '--settings', '--pythonpath',
-        '--no-color', '--force-color', '--skip-checks',
+        "--version",
+        "--verbosity",
+        "--traceback",
+        "--settings",
+        "--pythonpath",
+        "--no-color",
+        "--force-color",
+        "--skip-checks",
     }
 
     def _reordered_actions(self, actions):
         return sorted(
-            actions,
-            key=lambda a: set(a.option_strings) & self.show_last != set()
+            actions, key=lambda a: set(a.option_strings) & self.show_last != set()
         )
 
     def add_usage(self, usage, actions, *args, **kwargs):
@@ -115,6 +137,7 @@ class OutputWrapper(TextIOBase):
     """
     Wrapper around stdout/stderr
     """
+
     @property
     def style_func(self):
         return self._style_func
@@ -126,7 +149,7 @@ class OutputWrapper(TextIOBase):
         else:
             self._style_func = lambda x: x
 
-    def __init__(self, out, ending='\n'):
+    def __init__(self, out, ending="\n"):
         self._out = out
         self.style_func = None
         self.ending = ending
@@ -134,10 +157,14 @@ class OutputWrapper(TextIOBase):
     def __getattr__(self, name):
         return getattr(self._out, name)
 
-    def isatty(self):
-        return hasattr(self._out, 'isatty') and self._out.isatty()
+    def flush(self):
+        if hasattr(self._out, "flush"):
+            self._out.flush()
 
-    def write(self, msg, style_func=None, ending=None):
+    def isatty(self):
+        return hasattr(self._out, "isatty") and self._out.isatty()
+
+    def write(self, msg="", style_func=None, ending=None):
         ending = self.ending if ending is None else ending
         if ending and not msg.endswith(ending):
             msg += ending
@@ -201,8 +228,11 @@ class BaseCommand:
         migrations on disk don't match the migrations in the database.
 
     ``requires_system_checks``
-        A boolean; if ``True``, entire Django project will be checked for errors
-        prior to executing the command. Default value is ``True``.
+        A list or tuple of tags, e.g. [Tags.staticfiles, Tags.models]. System
+        checks registered in the chosen tags will be checked for errors prior
+        to executing the command. The value '__all__' can be used to specify
+        that all system checks should be performed. Default value is '__all__'.
+
         To validate an individual application's models
         rather than all applications' models, call
         ``self.check(app_configs)`` from ``handle()``, where ``app_configs``
@@ -213,19 +243,21 @@ class BaseCommand:
         A tuple of any options the command uses which aren't defined by the
         argument parser.
     """
+
     # Metadata about this command.
-    help = ''
+    help = ""
 
     # Configuration shortcuts that alter various logic.
     _called_from_command_line = False
     output_transaction = False  # Whether to wrap the output in a "BEGIN; COMMIT;"
     requires_migrations_checks = False
-    requires_system_checks = True
+    requires_system_checks = "__all__"
     # Arguments, common to all commands, which aren't defined by the argument
     # parser.
-    base_stealth_options = ('stderr', 'stdout')
+    base_stealth_options = ("stderr", "stdout")
     # Command-specific options not defined by the argument parser.
     stealth_options = ()
+    suppressed_base_arguments = set()
 
     def __init__(self, stdout=None, stderr=None, no_color=False, force_color=False):
         self.stdout = OutputWrapper(stdout or sys.stdout)
@@ -237,6 +269,21 @@ class BaseCommand:
         else:
             self.style = color_style(force_color)
             self.stderr.style_func = self.style.ERROR
+        if self.requires_system_checks in [False, True]:
+            warnings.warn(
+                "Using a boolean value for requires_system_checks is "
+                "deprecated. Use '__all__' instead of True, and [] (an empty "
+                "list) instead of False.",
+                RemovedInDjango41Warning,
+            )
+            self.requires_system_checks = (
+                ALL_CHECKS if self.requires_system_checks else []
+            )
+        if (
+            not isinstance(self.requires_system_checks, (list, tuple))
+            and self.requires_system_checks != ALL_CHECKS
+        ):
+            raise TypeError("requires_system_checks must be a list or tuple.")
 
     def get_version(self):
         """
@@ -252,44 +299,72 @@ class BaseCommand:
         parse the arguments to this command.
         """
         parser = CommandParser(
-            prog='%s %s' % (os.path.basename(prog_name), subcommand),
+            prog="%s %s" % (os.path.basename(prog_name), subcommand),
             description=self.help or None,
             formatter_class=DjangoHelpFormatter,
-            missing_args_message=getattr(self, 'missing_args_message', None),
-            called_from_command_line=getattr(self, '_called_from_command_line', None),
-            **kwargs
+            missing_args_message=getattr(self, "missing_args_message", None),
+            called_from_command_line=getattr(self, "_called_from_command_line", None),
+            **kwargs,
         )
-        parser.add_argument('--version', action='version', version=self.get_version())
-        parser.add_argument(
-            '-v', '--verbosity', default=1,
-            type=int, choices=[0, 1, 2, 3],
-            help='Verbosity level; 0=minimal output, 1=normal output, 2=verbose output, 3=very verbose output',
+        self.add_base_argument(
+            parser,
+            "--version",
+            action="version",
+            version=self.get_version(),
+            help="Show program's version number and exit.",
         )
-        parser.add_argument(
-            '--settings',
+        self.add_base_argument(
+            parser,
+            "-v",
+            "--verbosity",
+            default=1,
+            type=int,
+            choices=[0, 1, 2, 3],
             help=(
-                'The Python path to a settings module, e.g. '
-                '"myproject.settings.main". If this isn\'t provided, the '
-                'DJANGO_SETTINGS_MODULE environment variable will be used.'
+                "Verbosity level; 0=minimal output, 1=normal output, 2=verbose output, "
+                "3=very verbose output"
             ),
         )
-        parser.add_argument(
-            '--pythonpath',
-            help='A directory to add to the Python path, e.g. "/home/djangoprojects/myproject".',
+        self.add_base_argument(
+            parser,
+            "--settings",
+            help=(
+                "The Python path to a settings module, e.g. "
+                '"myproject.settings.main". If this isn\'t provided, the '
+                "DJANGO_SETTINGS_MODULE environment variable will be used."
+            ),
         )
-        parser.add_argument('--traceback', action='store_true', help='Raise on CommandError exceptions')
-        parser.add_argument(
-            '--no-color', action='store_true',
+        self.add_base_argument(
+            parser,
+            "--pythonpath",
+            help=(
+                "A directory to add to the Python path, e.g. "
+                '"/home/djangoprojects/myproject".'
+            ),
+        )
+        self.add_base_argument(
+            parser,
+            "--traceback",
+            action="store_true",
+            help="Raise on CommandError exceptions.",
+        )
+        self.add_base_argument(
+            parser,
+            "--no-color",
+            action="store_true",
             help="Don't colorize the command output.",
         )
-        parser.add_argument(
-            '--force-color', action='store_true',
-            help='Force colorization of the command output.',
+        self.add_base_argument(
+            parser,
+            "--force-color",
+            action="store_true",
+            help="Force colorization of the command output.",
         )
         if self.requires_system_checks:
             parser.add_argument(
-                '--skip-checks', action='store_true',
-                help='Skip system checks.',
+                "--skip-checks",
+                action="store_true",
+                help="Skip system checks.",
             )
         self.add_arguments(parser)
         return parser
@@ -300,6 +375,17 @@ class BaseCommand:
         """
         pass
 
+    def add_base_argument(self, parser, *args, **kwargs):
+        """
+        Call the parser's add_argument() method, suppressing the help text
+        according to BaseCommand.suppressed_base_arguments.
+        """
+        for arg in args:
+            if arg in self.suppressed_base_arguments:
+                kwargs["help"] = argparse.SUPPRESS
+                break
+        parser.add_argument(*args, **kwargs)
+
     def print_help(self, prog_name, subcommand):
         """
         Print the help message for this command, derived from
@@ -322,20 +408,20 @@ class BaseCommand:
         options = parser.parse_args(argv[2:])
         cmd_options = vars(options)
         # Move positional args out of options to mimic legacy optparse
-        args = cmd_options.pop('args', ())
+        args = cmd_options.pop("args", ())
         handle_default_options(options)
         try:
             self.execute(*args, **cmd_options)
-        except Exception as e:
-            if options.traceback or not isinstance(e, CommandError):
+        except CommandError as e:
+            if options.traceback:
                 raise
 
             # SystemCheckError takes care of its own formatting.
             if isinstance(e, SystemCheckError):
                 self.stderr.write(str(e), lambda x: x)
             else:
-                self.stderr.write('%s: %s' % (e.__class__.__name__, e))
-            sys.exit(1)
+                self.stderr.write("%s: %s" % (e.__class__.__name__, e))
+            sys.exit(e.returncode)
         finally:
             try:
                 connections.close_all()
@@ -350,27 +436,32 @@ class BaseCommand:
         controlled by the ``requires_system_checks`` attribute, except if
         force-skipped).
         """
-        if options['force_color'] and options['no_color']:
-            raise CommandError("The --no-color and --force-color options can't be used together.")
-        if options['force_color']:
+        if options["force_color"] and options["no_color"]:
+            raise CommandError(
+                "The --no-color and --force-color options can't be used together."
+            )
+        if options["force_color"]:
             self.style = color_style(force_color=True)
-        elif options['no_color']:
+        elif options["no_color"]:
             self.style = no_style()
             self.stderr.style_func = None
-        if options.get('stdout'):
-            self.stdout = OutputWrapper(options['stdout'])
-        if options.get('stderr'):
-            self.stderr = OutputWrapper(options['stderr'])
+        if options.get("stdout"):
+            self.stdout = OutputWrapper(options["stdout"])
+        if options.get("stderr"):
+            self.stderr = OutputWrapper(options["stderr"])
 
-        if self.requires_system_checks and not options['skip_checks']:
-            self.check()
+        if self.requires_system_checks and not options["skip_checks"]:
+            if self.requires_system_checks == ALL_CHECKS:
+                self.check()
+            else:
+                self.check(tags=self.requires_system_checks)
         if self.requires_migrations_checks:
             self.check_migrations()
         output = self.handle(*args, **options)
         if output:
             if self.output_transaction:
-                connection = connections[options.get('database', DEFAULT_DB_ALIAS)]
-                output = '%s\n%s\n%s' % (
+                connection = connections[options.get("database", DEFAULT_DB_ALIAS)]
+                output = "%s\n%s\n%s" % (
                     self.style.SQL_KEYWORD(connection.ops.start_transaction_sql()),
                     output,
                     self.style.SQL_KEYWORD(connection.ops.end_transaction_sql()),
@@ -378,38 +469,61 @@ class BaseCommand:
             self.stdout.write(output)
         return output
 
-    def _run_checks(self, **kwargs):
-        return checks.run_checks(**kwargs)
-
-    def check(self, app_configs=None, tags=None, display_num_errors=False,
-              include_deployment_checks=False, fail_level=checks.ERROR):
+    def check(
+        self,
+        app_configs=None,
+        tags=None,
+        display_num_errors=False,
+        include_deployment_checks=False,
+        fail_level=checks.ERROR,
+        databases=None,
+    ):
         """
         Use the system check framework to validate entire Django project.
         Raise CommandError for any serious message (error or critical errors).
         If there are only light messages (like warnings), print them to stderr
         and don't raise an exception.
         """
-        all_issues = self._run_checks(
+        all_issues = checks.run_checks(
             app_configs=app_configs,
             tags=tags,
             include_deployment_checks=include_deployment_checks,
+            databases=databases,
         )
 
         header, body, footer = "", "", ""
         visible_issue_count = 0  # excludes silenced warnings
 
         if all_issues:
-            debugs = [e for e in all_issues if e.level < checks.INFO and not e.is_silenced()]
-            infos = [e for e in all_issues if checks.INFO <= e.level < checks.WARNING and not e.is_silenced()]
-            warnings = [e for e in all_issues if checks.WARNING <= e.level < checks.ERROR and not e.is_silenced()]
-            errors = [e for e in all_issues if checks.ERROR <= e.level < checks.CRITICAL and not e.is_silenced()]
-            criticals = [e for e in all_issues if checks.CRITICAL <= e.level and not e.is_silenced()]
+            debugs = [
+                e for e in all_issues if e.level < checks.INFO and not e.is_silenced()
+            ]
+            infos = [
+                e
+                for e in all_issues
+                if checks.INFO <= e.level < checks.WARNING and not e.is_silenced()
+            ]
+            warnings = [
+                e
+                for e in all_issues
+                if checks.WARNING <= e.level < checks.ERROR and not e.is_silenced()
+            ]
+            errors = [
+                e
+                for e in all_issues
+                if checks.ERROR <= e.level < checks.CRITICAL and not e.is_silenced()
+            ]
+            criticals = [
+                e
+                for e in all_issues
+                if checks.CRITICAL <= e.level and not e.is_silenced()
+            ]
             sorted_issues = [
-                (criticals, 'CRITICALS'),
-                (errors, 'ERRORS'),
-                (warnings, 'WARNINGS'),
-                (infos, 'INFOS'),
-                (debugs, 'DEBUGS'),
+                (criticals, "CRITICALS"),
+                (errors, "ERRORS"),
+                (warnings, "WARNINGS"),
+                (infos, "INFOS"),
+                (debugs, "DEBUGS"),
             ]
 
             for issues, group_name in sorted_issues:
@@ -419,20 +533,23 @@ class BaseCommand:
                         self.style.ERROR(str(e))
                         if e.is_serious()
                         else self.style.WARNING(str(e))
-                        for e in issues)
+                        for e in issues
+                    )
                     formatted = "\n".join(sorted(formatted))
-                    body += '\n%s:\n%s\n' % (group_name, formatted)
+                    body += "\n%s:\n%s\n" % (group_name, formatted)
 
         if visible_issue_count:
             header = "System check identified some issues:\n"
 
         if display_num_errors:
             if visible_issue_count:
-                footer += '\n'
+                footer += "\n"
             footer += "System check identified %s (%s silenced)." % (
-                "no issues" if visible_issue_count == 0 else
-                "1 issue" if visible_issue_count == 1 else
-                "%s issues" % visible_issue_count,
+                "no issues"
+                if visible_issue_count == 0
+                else "1 issue"
+                if visible_issue_count == 1
+                else "%s issues" % visible_issue_count,
                 len(all_issues) - visible_issue_count,
             )
 
@@ -454,6 +571,7 @@ class BaseCommand:
         migrations in the database.
         """
         from django.db.migrations.executor import MigrationExecutor
+
         try:
             executor = MigrationExecutor(connections[DEFAULT_DB_ALIAS])
         except ImproperlyConfigured:
@@ -462,25 +580,32 @@ class BaseCommand:
 
         plan = executor.migration_plan(executor.loader.graph.leaf_nodes())
         if plan:
-            apps_waiting_migration = sorted({migration.app_label for migration, backwards in plan})
+            apps_waiting_migration = sorted(
+                {migration.app_label for migration, backwards in plan}
+            )
             self.stdout.write(
                 self.style.NOTICE(
                     "\nYou have %(unapplied_migration_count)s unapplied migration(s). "
                     "Your project may not work properly until you apply the "
-                    "migrations for app(s): %(apps_waiting_migration)s." % {
+                    "migrations for app(s): %(apps_waiting_migration)s."
+                    % {
                         "unapplied_migration_count": len(plan),
                         "apps_waiting_migration": ", ".join(apps_waiting_migration),
                     }
                 )
             )
-            self.stdout.write(self.style.NOTICE("Run 'python manage.py migrate' to apply them.\n"))
+            self.stdout.write(
+                self.style.NOTICE("Run 'python manage.py migrate' to apply them.")
+            )
 
     def handle(self, *args, **options):
         """
         The actual logic of the command. Subclasses must implement
         this method.
         """
-        raise NotImplementedError('subclasses of BaseCommand must provide a handle() method')
+        raise NotImplementedError(
+            "subclasses of BaseCommand must provide a handle() method"
+        )
 
 
 class AppCommand(BaseCommand):
@@ -491,23 +616,32 @@ class AppCommand(BaseCommand):
     Rather than implementing ``handle()``, subclasses must implement
     ``handle_app_config()``, which will be called once for each application.
     """
+
     missing_args_message = "Enter at least one application label."
 
     def add_arguments(self, parser):
-        parser.add_argument('args', metavar='app_label', nargs='+', help='One or more application label.')
+        parser.add_argument(
+            "args",
+            metavar="app_label",
+            nargs="+",
+            help="One or more application label.",
+        )
 
     def handle(self, *app_labels, **options):
         from django.apps import apps
+
         try:
             app_configs = [apps.get_app_config(app_label) for app_label in app_labels]
         except (LookupError, ImportError) as e:
-            raise CommandError("%s. Are you sure your INSTALLED_APPS setting is correct?" % e)
+            raise CommandError(
+                "%s. Are you sure your INSTALLED_APPS setting is correct?" % e
+            )
         output = []
         for app_config in app_configs:
             app_output = self.handle_app_config(app_config, **options)
             if app_output:
                 output.append(app_output)
-        return '\n'.join(output)
+        return "\n".join(output)
 
     def handle_app_config(self, app_config, **options):
         """
@@ -515,8 +649,8 @@ class AppCommand(BaseCommand):
         corresponding to an application label given on the command line.
         """
         raise NotImplementedError(
-            "Subclasses of AppCommand must provide"
-            "a handle_app_config() method.")
+            "Subclasses of AppCommand must provide a handle_app_config() method."
+        )
 
 
 class LabelCommand(BaseCommand):
@@ -531,11 +665,12 @@ class LabelCommand(BaseCommand):
     If the arguments should be names of installed applications, use
     ``AppCommand`` instead.
     """
-    label = 'label'
+
+    label = "label"
     missing_args_message = "Enter at least one %s." % label
 
     def add_arguments(self, parser):
-        parser.add_argument('args', metavar=self.label, nargs='+')
+        parser.add_argument("args", metavar=self.label, nargs="+")
 
     def handle(self, *labels, **options):
         output = []
@@ -543,11 +678,13 @@ class LabelCommand(BaseCommand):
             label_output = self.handle_label(label, **options)
             if label_output:
                 output.append(label_output)
-        return '\n'.join(output)
+        return "\n".join(output)
 
     def handle_label(self, label, **options):
         """
         Perform the command's actions for ``label``, which will be the
         string as given on the command line.
         """
-        raise NotImplementedError('subclasses of LabelCommand must provide a handle_label() method')
+        raise NotImplementedError(
+            "subclasses of LabelCommand must provide a handle_label() method"
+        )
