@@ -1,14 +1,10 @@
+import sys
 import time
 from importlib import import_module
 
 from django.apps import apps
-from django.core.checks import Tags, run_checks
-from django.core.management.base import (
-    BaseCommand, CommandError, no_translations,
-)
-from django.core.management.sql import (
-    emit_post_migrate_signal, emit_pre_migrate_signal,
-)
+from django.core.management.base import BaseCommand, CommandError, no_translations
+from django.core.management.sql import emit_post_migrate_signal, emit_pre_migrate_signal
 from django.db import DEFAULT_DB_ALIAS, connections, router
 from django.db.migrations.autodetector import MigrationAutodetector
 from django.db.migrations.executor import MigrationExecutor
@@ -19,66 +15,92 @@ from django.utils.text import Truncator
 
 
 class Command(BaseCommand):
-    help = "Updates database schema. Manages both apps with migrations and those without."
+    help = (
+        "Updates database schema. Manages both apps with migrations and those without."
+    )
+    requires_system_checks = []
 
     def add_arguments(self, parser):
         parser.add_argument(
-            'app_label', nargs='?',
-            help='App label of an application to synchronize the state.',
+            "--skip-checks",
+            action="store_true",
+            help="Skip system checks.",
         )
         parser.add_argument(
-            'migration_name', nargs='?',
-            help='Database state will be brought to the state after that '
-                 'migration. Use the name "zero" to unapply all migrations.',
+            "app_label",
+            nargs="?",
+            help="App label of an application to synchronize the state.",
         )
         parser.add_argument(
-            '--noinput', '--no-input', action='store_false', dest='interactive',
-            help='Tells Django to NOT prompt the user for input of any kind.',
+            "migration_name",
+            nargs="?",
+            help="Database state will be brought to the state after that "
+            'migration. Use the name "zero" to unapply all migrations.',
         )
         parser.add_argument(
-            '--database',
+            "--noinput",
+            "--no-input",
+            action="store_false",
+            dest="interactive",
+            help="Tells Django to NOT prompt the user for input of any kind.",
+        )
+        parser.add_argument(
+            "--database",
             default=DEFAULT_DB_ALIAS,
-            help='Nominates a database to synchronize. Defaults to the "default" database.',
+            help=(
+                'Nominates a database to synchronize. Defaults to the "default" '
+                "database."
+            ),
         )
         parser.add_argument(
-            '--fake', action='store_true',
-            help='Mark migrations as run without actually running them.',
+            "--fake",
+            action="store_true",
+            help="Mark migrations as run without actually running them.",
         )
         parser.add_argument(
-            '--fake-initial', action='store_true',
-            help='Detect if tables already exist and fake-apply initial migrations if so. Make sure '
-                 'that the current database schema matches your initial migration before using this '
-                 'flag. Django will only check for an existing table name.',
+            "--fake-initial",
+            action="store_true",
+            help=(
+                "Detect if tables already exist and fake-apply initial migrations if "
+                "so. Make sure that the current database schema matches your initial "
+                "migration before using this flag. Django will only check for an "
+                "existing table name."
+            ),
         )
         parser.add_argument(
-            '--plan', action='store_true',
-            help='Shows a list of the migration actions that will be performed.',
+            "--plan",
+            action="store_true",
+            help="Shows a list of the migration actions that will be performed.",
         )
         parser.add_argument(
-            '--run-syncdb', action='store_true',
-            help='Creates tables for apps without migrations.',
+            "--run-syncdb",
+            action="store_true",
+            help="Creates tables for apps without migrations.",
+        )
+        parser.add_argument(
+            "--check",
+            action="store_true",
+            dest="check_unapplied",
+            help="Exits with a non-zero status if unapplied migrations exist.",
         )
-
-    def _run_checks(self, **kwargs):
-        issues = run_checks(tags=[Tags.database])
-        issues.extend(super()._run_checks(**kwargs))
-        return issues
 
     @no_translations
     def handle(self, *args, **options):
+        database = options["database"]
+        if not options["skip_checks"]:
+            self.check(databases=[database])
 
-        self.verbosity = options['verbosity']
-        self.interactive = options['interactive']
+        self.verbosity = options["verbosity"]
+        self.interactive = options["interactive"]
 
         # Import the 'management' module within each installed app, to register
         # dispatcher events.
         for app_config in apps.get_app_configs():
             if module_has_submodule(app_config.module, "management"):
-                import_module('.management', app_config.name)
+                import_module(".management", app_config.name)
 
         # Get the database we're operating from
-        db = options['database']
-        connection = connections[db]
+        connection = connections[database]
 
         # Hook for backends needing any database preparation
         connection.prepare_database()
@@ -93,8 +115,7 @@ class Command(BaseCommand):
         conflicts = executor.loader.detect_conflicts()
         if conflicts:
             name_str = "; ".join(
-                "%s in %s" % (", ".join(names), app)
-                for app, names in conflicts.items()
+                "%s in %s" % (", ".join(names), app) for app, names in conflicts.items()
             )
             raise CommandError(
                 "Conflicting migrations detected; multiple leaf nodes in the "
@@ -103,100 +124,132 @@ class Command(BaseCommand):
             )
 
         # If they supplied command line arguments, work out what they mean.
-        run_syncdb = options['run_syncdb']
+        run_syncdb = options["run_syncdb"]
         target_app_labels_only = True
-        if options['app_label']:
+        if options["app_label"]:
             # Validate app_label.
-            app_label = options['app_label']
+            app_label = options["app_label"]
             try:
                 apps.get_app_config(app_label)
             except LookupError as err:
                 raise CommandError(str(err))
             if run_syncdb:
                 if app_label in executor.loader.migrated_apps:
-                    raise CommandError("Can't use run_syncdb with app '%s' as it has migrations." % app_label)
+                    raise CommandError(
+                        "Can't use run_syncdb with app '%s' as it has migrations."
+                        % app_label
+                    )
             elif app_label not in executor.loader.migrated_apps:
                 raise CommandError("App '%s' does not have migrations." % app_label)
 
-        if options['app_label'] and options['migration_name']:
-            migration_name = options['migration_name']
+        if options["app_label"] and options["migration_name"]:
+            migration_name = options["migration_name"]
             if migration_name == "zero":
                 targets = [(app_label, None)]
             else:
                 try:
-                    migration = executor.loader.get_migration_by_prefix(app_label, migration_name)
+                    migration = executor.loader.get_migration_by_prefix(
+                        app_label, migration_name
+                    )
                 except AmbiguityError:
                     raise CommandError(
                         "More than one migration matches '%s' in app '%s'. "
-                        "Please be more specific." %
-                        (migration_name, app_label)
+                        "Please be more specific." % (migration_name, app_label)
                     )
                 except KeyError:
-                    raise CommandError("Cannot find a migration matching '%s' from app '%s'." % (
-                        migration_name, app_label))
-                targets = [(app_label, migration.name)]
+                    raise CommandError(
+                        "Cannot find a migration matching '%s' from app '%s'."
+                        % (migration_name, app_label)
+                    )
+                target = (app_label, migration.name)
+                # Partially applied squashed migrations are not included in the
+                # graph, use the last replacement instead.
+                if (
+                    target not in executor.loader.graph.nodes
+                    and target in executor.loader.replacements
+                ):
+                    incomplete_migration = executor.loader.replacements[target]
+                    target = incomplete_migration.replaces[-1]
+                targets = [target]
             target_app_labels_only = False
-        elif options['app_label']:
-            targets = [key for key in executor.loader.graph.leaf_nodes() if key[0] == app_label]
+        elif options["app_label"]:
+            targets = [
+                key for key in executor.loader.graph.leaf_nodes() if key[0] == app_label
+            ]
         else:
             targets = executor.loader.graph.leaf_nodes()
 
         plan = executor.migration_plan(targets)
+        exit_dry = plan and options["check_unapplied"]
 
-        if options['plan']:
-            self.stdout.write('Planned operations:', self.style.MIGRATE_LABEL)
+        if options["plan"]:
+            self.stdout.write("Planned operations:", self.style.MIGRATE_LABEL)
             if not plan:
-                self.stdout.write('  No planned migration operations.')
+                self.stdout.write("  No planned migration operations.")
             for migration, backwards in plan:
                 self.stdout.write(str(migration), self.style.MIGRATE_HEADING)
                 for operation in migration.operations:
                     message, is_error = self.describe_operation(operation, backwards)
                     style = self.style.WARNING if is_error else None
-                    self.stdout.write('    ' + message, style)
+                    self.stdout.write("    " + message, style)
+            if exit_dry:
+                sys.exit(1)
             return
+        if exit_dry:
+            sys.exit(1)
 
         # At this point, ignore run_syncdb if there aren't any apps to sync.
-        run_syncdb = options['run_syncdb'] and executor.loader.unmigrated_apps
+        run_syncdb = options["run_syncdb"] and executor.loader.unmigrated_apps
         # Print some useful info
         if self.verbosity >= 1:
             self.stdout.write(self.style.MIGRATE_HEADING("Operations to perform:"))
             if run_syncdb:
-                if options['app_label']:
+                if options["app_label"]:
                     self.stdout.write(
-                        self.style.MIGRATE_LABEL("  Synchronize unmigrated app: %s" % app_label)
+                        self.style.MIGRATE_LABEL(
+                            "  Synchronize unmigrated app: %s" % app_label
+                        )
                     )
                 else:
                     self.stdout.write(
-                        self.style.MIGRATE_LABEL("  Synchronize unmigrated apps: ") +
-                        (", ".join(sorted(executor.loader.unmigrated_apps)))
+                        self.style.MIGRATE_LABEL("  Synchronize unmigrated apps: ")
+                        + (", ".join(sorted(executor.loader.unmigrated_apps)))
                     )
             if target_app_labels_only:
                 self.stdout.write(
-                    self.style.MIGRATE_LABEL("  Apply all migrations: ") +
-                    (", ".join(sorted({a for a, n in targets})) or "(none)")
+                    self.style.MIGRATE_LABEL("  Apply all migrations: ")
+                    + (", ".join(sorted({a for a, n in targets})) or "(none)")
                 )
             else:
                 if targets[0][1] is None:
-                    self.stdout.write(self.style.MIGRATE_LABEL(
-                        "  Unapply all migrations: ") + "%s" % (targets[0][0],)
+                    self.stdout.write(
+                        self.style.MIGRATE_LABEL("  Unapply all migrations: ")
+                        + str(targets[0][0])
                     )
                 else:
-                    self.stdout.write(self.style.MIGRATE_LABEL(
-                        "  Target specific migration: ") + "%s, from %s"
-                        % (targets[0][1], targets[0][0])
+                    self.stdout.write(
+                        self.style.MIGRATE_LABEL("  Target specific migration: ")
+                        + "%s, from %s" % (targets[0][1], targets[0][0])
                     )
 
         pre_migrate_state = executor._create_project_state(with_applied_migrations=True)
         pre_migrate_apps = pre_migrate_state.apps
         emit_pre_migrate_signal(
-            self.verbosity, self.interactive, connection.alias, apps=pre_migrate_apps, plan=plan,
+            self.verbosity,
+            self.interactive,
+            connection.alias,
+            stdout=self.stdout,
+            apps=pre_migrate_apps,
+            plan=plan,
         )
 
         # Run the syncdb phase.
         if run_syncdb:
             if self.verbosity >= 1:
-                self.stdout.write(self.style.MIGRATE_HEADING("Synchronizing apps without migrations:"))
-            if options['app_label']:
+                self.stdout.write(
+                    self.style.MIGRATE_HEADING("Synchronizing apps without migrations:")
+                )
+            if options["app_label"]:
                 self.sync_apps(connection, [app_label])
             else:
                 self.sync_apps(connection, executor.loader.unmigrated_apps)
@@ -207,29 +260,38 @@ class Command(BaseCommand):
         if not plan:
             if self.verbosity >= 1:
                 self.stdout.write("  No migrations to apply.")
-                # If there's changes that aren't in migrations yet, tell them how to fix it.
+                # If there's changes that aren't in migrations yet, tell them
+                # how to fix it.
                 autodetector = MigrationAutodetector(
                     executor.loader.project_state(),
                     ProjectState.from_apps(apps),
                 )
                 changes = autodetector.changes(graph=executor.loader.graph)
                 if changes:
-                    self.stdout.write(self.style.NOTICE(
-                        "  Your models have changes that are not yet reflected "
-                        "in a migration, and so won't be applied."
-                    ))
-                    self.stdout.write(self.style.NOTICE(
-                        "  Run 'manage.py makemigrations' to make new "
-                        "migrations, and then re-run 'manage.py migrate' to "
-                        "apply them."
-                    ))
+                    self.stdout.write(
+                        self.style.NOTICE(
+                            "  Your models in app(s): %s have changes that are not "
+                            "yet reflected in a migration, and so won't be "
+                            "applied." % ", ".join(repr(app) for app in sorted(changes))
+                        )
+                    )
+                    self.stdout.write(
+                        self.style.NOTICE(
+                            "  Run 'manage.py makemigrations' to make new "
+                            "migrations, and then re-run 'manage.py migrate' to "
+                            "apply them."
+                        )
+                    )
             fake = False
             fake_initial = False
         else:
-            fake = options['fake']
-            fake_initial = options['fake_initial']
+            fake = options["fake"]
+            fake_initial = options["fake_initial"]
         post_migrate_state = executor.migrate(
-            targets, plan=plan, state=pre_migrate_state.clone(), fake=fake,
+            targets,
+            plan=plan,
+            state=pre_migrate_state.clone(),
+            fake=fake,
             fake_initial=fake_initial,
         )
         # post_migrate signals have access to all models. Ensure that all models
@@ -246,14 +308,19 @@ class Command(BaseCommand):
                 model_key = model_state.app_label, model_state.name_lower
                 model_keys.append(model_key)
                 post_migrate_apps.unregister_model(*model_key)
-        post_migrate_apps.render_multiple([
-            ModelState.from_model(apps.get_model(*model)) for model in model_keys
-        ])
+        post_migrate_apps.render_multiple(
+            [ModelState.from_model(apps.get_model(*model)) for model in model_keys]
+        )
 
         # Send the post_migrate signal, so individual apps can do whatever they need
         # to do at this point.
         emit_post_migrate_signal(
-            self.verbosity, self.interactive, connection.alias, apps=post_migrate_apps, plan=plan,
+            self.verbosity,
+            self.interactive,
+            connection.alias,
+            stdout=self.stdout,
+            apps=post_migrate_apps,
+            plan=plan,
         )
 
     def migration_progress_callback(self, action, migration=None, fake=False):
@@ -265,7 +332,9 @@ class Command(BaseCommand):
                 self.stdout.write("  Applying %s..." % migration, ending="")
                 self.stdout.flush()
             elif action == "apply_success":
-                elapsed = " (%.3fs)" % (time.monotonic() - self.start) if compute_time else ""
+                elapsed = (
+                    " (%.3fs)" % (time.monotonic() - self.start) if compute_time else ""
+                )
                 if fake:
                     self.stdout.write(self.style.SUCCESS(" FAKED" + elapsed))
                 else:
@@ -276,7 +345,9 @@ class Command(BaseCommand):
                 self.stdout.write("  Unapplying %s..." % migration, ending="")
                 self.stdout.flush()
             elif action == "unapply_success":
-                elapsed = " (%.3fs)" % (time.monotonic() - self.start) if compute_time else ""
+                elapsed = (
+                    " (%.3fs)" % (time.monotonic() - self.start) if compute_time else ""
+                )
                 if fake:
                     self.stdout.write(self.style.SUCCESS(" FAKED" + elapsed))
                 else:
@@ -287,7 +358,9 @@ class Command(BaseCommand):
                 self.stdout.write("  Rendering model states...", ending="")
                 self.stdout.flush()
             elif action == "render_success":
-                elapsed = " (%.3fs)" % (time.monotonic() - self.start) if compute_time else ""
+                elapsed = (
+                    " (%.3fs)" % (time.monotonic() - self.start) if compute_time else ""
+                )
                 self.stdout.write(self.style.SUCCESS(" DONE" + elapsed))
 
     def sync_apps(self, connection, app_labels):
@@ -299,7 +372,9 @@ class Command(BaseCommand):
         all_models = [
             (
                 app_config.label,
-                router.get_migratable_models(app_config, connection.alias, include_auto_created=False),
+                router.get_migratable_models(
+                    app_config, connection.alias, include_auto_created=False
+                ),
             )
             for app_config in apps.get_app_configs()
             if app_config.models_module is not None and app_config.label in app_labels
@@ -309,8 +384,11 @@ class Command(BaseCommand):
             opts = model._meta
             converter = connection.introspection.identifier_converter
             return not (
-                (converter(opts.db_table) in tables) or
-                (opts.auto_created and converter(opts.auto_created._meta.db_table) in tables)
+                (converter(opts.db_table) in tables)
+                or (
+                    opts.auto_created
+                    and converter(opts.auto_created._meta.db_table) in tables
+                )
             )
 
         manifest = {
@@ -320,7 +398,7 @@ class Command(BaseCommand):
 
         # Create the tables for each model
         if self.verbosity >= 1:
-            self.stdout.write("  Creating tables...\n")
+            self.stdout.write("  Creating tables...")
         with connection.schema_editor() as editor:
             for app_name, model_list in manifest.items():
                 for model in model_list:
@@ -329,36 +407,39 @@ class Command(BaseCommand):
                         continue
                     if self.verbosity >= 3:
                         self.stdout.write(
-                            "    Processing %s.%s model\n" % (app_name, model._meta.object_name)
+                            "    Processing %s.%s model"
+                            % (app_name, model._meta.object_name)
                         )
                     if self.verbosity >= 1:
-                        self.stdout.write("    Creating table %s\n" % model._meta.db_table)
+                        self.stdout.write(
+                            "    Creating table %s" % model._meta.db_table
+                        )
                     editor.create_model(model)
 
             # Deferred SQL is executed when exiting the editor's context.
             if self.verbosity >= 1:
-                self.stdout.write("    Running deferred SQL...\n")
+                self.stdout.write("    Running deferred SQL...")
 
     @staticmethod
     def describe_operation(operation, backwards):
         """Return a string that describes a migration operation for --plan."""
-        prefix = ''
+        prefix = ""
         is_error = False
-        if hasattr(operation, 'code'):
+        if hasattr(operation, "code"):
             code = operation.reverse_code if backwards else operation.code
-            action = (code.__doc__ or '') if code else None
-        elif hasattr(operation, 'sql'):
+            action = (code.__doc__ or "") if code else None
+        elif hasattr(operation, "sql"):
             action = operation.reverse_sql if backwards else operation.sql
         else:
-            action = ''
+            action = ""
             if backwards:
-                prefix = 'Undo '
+                prefix = "Undo "
         if action is not None:
-            action = str(action).replace('\n', '')
+            action = str(action).replace("\n", "")
         elif backwards:
-            action = 'IRREVERSIBLE'
+            action = "IRREVERSIBLE"
             is_error = True
         if action:
-            action = ' -> ' + action
+            action = " -> " + action
         truncated = Truncator(action)
         return prefix + operation.describe() + truncated.chars(40), is_error
