@@ -1,3 +1,4 @@
+import argparse
 import cgi
 import mimetypes
 import os
@@ -28,38 +29,61 @@ class TemplateCommand(BaseCommand):
     :param directory: The directory to which the template should be copied.
     :param options: The additional variables passed to project or app templates
     """
-    requires_system_checks = False
+
+    requires_system_checks = []
     # The supported URL schemes
-    url_schemes = ['http', 'https', 'ftp']
+    url_schemes = ["http", "https", "ftp"]
     # Rewrite the following suffixes when determining the target filename.
     rewrite_template_suffixes = (
         # Allow shipping invalid .py files without byte-compilation.
-        ('.py-tpl', '.py'),
+        (".py-tpl", ".py"),
     )
 
     def add_arguments(self, parser):
-        parser.add_argument('name', help='Name of the application or project.')
-        parser.add_argument('directory', nargs='?', help='Optional destination directory')
-        parser.add_argument('--template', help='The path or URL to load the template from.')
+        parser.add_argument("name", help="Name of the application or project.")
         parser.add_argument(
-            '--extension', '-e', dest='extensions',
-            action='append', default=['py'],
-            help='The file extension(s) to render (default: "py"). '
-                 'Separate multiple extensions with commas, or use '
-                 '-e multiple times.'
+            "directory", nargs="?", help="Optional destination directory"
         )
         parser.add_argument(
-            '--name', '-n', dest='files',
-            action='append', default=[],
-            help='The file name(s) to render. Separate multiple file names '
-                 'with commas, or use -n multiple times.'
+            "--template", help="The path or URL to load the template from."
+        )
+        parser.add_argument(
+            "--extension",
+            "-e",
+            dest="extensions",
+            action="append",
+            default=["py"],
+            help='The file extension(s) to render (default: "py"). '
+            "Separate multiple extensions with commas, or use "
+            "-e multiple times.",
+        )
+        parser.add_argument(
+            "--name",
+            "-n",
+            dest="files",
+            action="append",
+            default=[],
+            help="The file name(s) to render. Separate multiple file names "
+            "with commas, or use -n multiple times.",
+        )
+        parser.add_argument(
+            "--exclude",
+            "-x",
+            action="append",
+            default=argparse.SUPPRESS,
+            nargs="?",
+            const="",
+            help=(
+                "The directory name(s) to exclude, in addition to .git and "
+                "__pycache__. Can be used multiple times."
+            ),
         )
 
     def handle(self, app_or_project, name, target=None, **options):
         self.app_or_project = app_or_project
-        self.a_or_an = 'an' if app_or_project == 'app' else 'a'
+        self.a_or_an = "an" if app_or_project == "app" else "a"
         self.paths_to_remove = []
-        self.verbosity = options['verbosity']
+        self.verbosity = options["verbosity"]
 
         self.validate_name(name)
 
@@ -73,48 +97,56 @@ class TemplateCommand(BaseCommand):
             except OSError as e:
                 raise CommandError(e)
         else:
-            if app_or_project == 'app':
-                self.validate_name(os.path.basename(target), 'directory')
             top_dir = os.path.abspath(os.path.expanduser(target))
+            if app_or_project == "app":
+                self.validate_name(os.path.basename(top_dir), "directory")
             if not os.path.exists(top_dir):
-                raise CommandError("Destination directory '%s' does not "
-                                   "exist, please create it first." % top_dir)
+                raise CommandError(
+                    "Destination directory '%s' does not "
+                    "exist, please create it first." % top_dir
+                )
 
-        extensions = tuple(handle_extensions(options['extensions']))
+        extensions = tuple(handle_extensions(options["extensions"]))
         extra_files = []
-        for file in options['files']:
-            extra_files.extend(map(lambda x: x.strip(), file.split(',')))
+        excluded_directories = [".git", "__pycache__"]
+        for file in options["files"]:
+            extra_files.extend(map(lambda x: x.strip(), file.split(",")))
+        if exclude := options.get("exclude"):
+            for directory in exclude:
+                excluded_directories.append(directory.strip())
         if self.verbosity >= 2:
             self.stdout.write(
-                'Rendering %s template files with extensions: %s'
-                % (app_or_project, ', '.join(extensions))
+                "Rendering %s template files with extensions: %s"
+                % (app_or_project, ", ".join(extensions))
             )
             self.stdout.write(
-                'Rendering %s template files with filenames: %s'
-                % (app_or_project, ', '.join(extra_files))
+                "Rendering %s template files with filenames: %s"
+                % (app_or_project, ", ".join(extra_files))
             )
-        base_name = '%s_name' % app_or_project
-        base_subdir = '%s_template' % app_or_project
-        base_directory = '%s_directory' % app_or_project
-        camel_case_name = 'camel_case_%s_name' % app_or_project
-        camel_case_value = ''.join(x for x in name.title() if x != '_')
+        base_name = "%s_name" % app_or_project
+        base_subdir = "%s_template" % app_or_project
+        base_directory = "%s_directory" % app_or_project
+        camel_case_name = "camel_case_%s_name" % app_or_project
+        camel_case_value = "".join(x for x in name.title() if x != "_")
 
-        context = Context({
-            **options,
-            base_name: name,
-            base_directory: top_dir,
-            camel_case_name: camel_case_value,
-            'docs_version': get_docs_version(),
-            'django_version': django.__version__,
-        }, autoescape=False)
+        context = Context(
+            {
+                **options,
+                base_name: name,
+                base_directory: top_dir,
+                camel_case_name: camel_case_value,
+                "docs_version": get_docs_version(),
+                "django_version": django.__version__,
+            },
+            autoescape=False,
+        )
 
         # Setup a stub settings environment for template rendering
         if not settings.configured:
             settings.configure()
             django.setup()
 
-        template_dir = self.handle_template(options['template'],
-                                            base_subdir)
+        template_dir = self.handle_template(options["template"], base_subdir)
         prefix_length = len(template_dir) + 1
 
         for root, dirs, files in os.walk(template_dir):
@@ -126,11 +158,14 @@ class TemplateCommand(BaseCommand):
                 os.makedirs(target_dir, exist_ok=True)
 
             for dirname in dirs[:]:
-                if dirname.startswith('.') or dirname == '__pycache__':
+                if "exclude" not in options:
+                    if dirname.startswith(".") or dirname == "__pycache__":
+                        dirs.remove(dirname)
+                elif dirname in excluded_directories:
                     dirs.remove(dirname)
 
             for filename in files:
-                if filename.endswith(('.pyo', '.pyc', '.py.class')):
+                if filename.endswith((".pyo", ".pyc", ".py.class")):
                     # Ignore some files as they cause various breakages.
                     continue
                 old_path = os.path.join(root, filename)
@@ -139,31 +174,34 @@ class TemplateCommand(BaseCommand):
                 )
                 for old_suffix, new_suffix in self.rewrite_template_suffixes:
                     if new_path.endswith(old_suffix):
-                        new_path = new_path[:-len(old_suffix)] + new_suffix
+                        new_path = new_path[: -len(old_suffix)] + new_suffix
                         break  # Only rewrite once
 
                 if os.path.exists(new_path):
                     raise CommandError(
                         "%s already exists. Overlaying %s %s into an existing "
-                        "directory won't replace conflicting files." % (
-                            new_path, self.a_or_an, app_or_project,
+                        "directory won't replace conflicting files."
+                        % (
+                            new_path,
+                            self.a_or_an,
+                            app_or_project,
                         )
                     )
 
                 # Only render the Python files, as we don't want to
                 # accidentally render Django templates files
                 if new_path.endswith(extensions) or filename in extra_files:
-                    with open(old_path, encoding='utf-8') as template_file:
+                    with open(old_path, encoding="utf-8") as template_file:
                         content = template_file.read()
                     template = Engine().from_string(content)
                     content = template.render(context)
-                    with open(new_path, 'w', encoding='utf-8') as new_file:
+                    with open(new_path, "w", encoding="utf-8") as new_file:
                         new_file.write(content)
                 else:
                     shutil.copyfile(old_path, new_path)
 
                 if self.verbosity >= 2:
-                    self.stdout.write('Creating %s' % new_path)
+                    self.stdout.write("Creating %s" % new_path)
                 try:
                     shutil.copymode(old_path, new_path)
                     self.make_writeable(new_path)
@@ -171,11 +209,13 @@ class TemplateCommand(BaseCommand):
                     self.stderr.write(
                         "Notice: Couldn't set permission bits on %s. You're "
                         "probably using an uncommon filesystem setup. No "
-                        "problem." % new_path, self.style.NOTICE)
+                        "problem." % new_path,
+                        self.style.NOTICE,
+                    )
 
         if self.paths_to_remove:
             if self.verbosity >= 2:
-                self.stdout.write('Cleaning up temporary files.')
+                self.stdout.write("Cleaning up temporary files.")
             for path_to_remove in self.paths_to_remove:
                 if os.path.isfile(path_to_remove):
                     os.remove(path_to_remove)
@@ -189,9 +229,9 @@ class TemplateCommand(BaseCommand):
         directory isn't known.
         """
         if template is None:
-            return os.path.join(django.__path__[0], 'conf', subdir)
+            return os.path.join(django.__path__[0], "conf", subdir)
         else:
-            if template.startswith('file://'):
+            if template.startswith("file://"):
                 template = template[7:]
             expanded_template = os.path.expanduser(template)
             expanded_template = os.path.normpath(expanded_template)
@@ -205,15 +245,18 @@ class TemplateCommand(BaseCommand):
             if os.path.exists(absolute_path):
                 return self.extract(absolute_path)
 
-        raise CommandError("couldn't handle %s template %s." %
-                           (self.app_or_project, template))
+        raise CommandError(
+            "couldn't handle %s template %s." % (self.app_or_project, template)
+        )
 
-    def validate_name(self, name, name_or_dir='name'):
+    def validate_name(self, name, name_or_dir="name"):
         if name is None:
-            raise CommandError('you must provide {an} {app} name'.format(
-                an=self.a_or_an,
-                app=self.app_or_project,
-            ))
+            raise CommandError(
+                "you must provide {an} {app} name".format(
+                    an=self.a_or_an,
+                    app=self.app_or_project,
+                )
+            )
         # Check it's a valid directory name.
         if not name.isidentifier():
             raise CommandError(
@@ -245,41 +288,43 @@ class TemplateCommand(BaseCommand):
         """
         Download the given URL and return the file name.
         """
+
         def cleanup_url(url):
-            tmp = url.rstrip('/')
-            filename = tmp.split('/')[-1]
-            if url.endswith('/'):
-                display_url = tmp + '/'
+            tmp = url.rstrip("/")
+            filename = tmp.split("/")[-1]
+            if url.endswith("/"):
+                display_url = tmp + "/"
             else:
                 display_url = url
             return filename, display_url
 
-        prefix = 'django_%s_template_' % self.app_or_project
-        tempdir = tempfile.mkdtemp(prefix=prefix, suffix='_download')
+        prefix = "django_%s_template_" % self.app_or_project
+        tempdir = tempfile.mkdtemp(prefix=prefix, suffix="_download")
         self.paths_to_remove.append(tempdir)
         filename, display_url = cleanup_url(url)
 
         if self.verbosity >= 2:
-            self.stdout.write('Downloading %s' % display_url)
+            self.stdout.write("Downloading %s" % display_url)
         try:
             the_path, info = urlretrieve(url, os.path.join(tempdir, filename))
         except OSError as e:
-            raise CommandError("couldn't download URL %s to %s: %s" %
-                               (url, filename, e))
+            raise CommandError(
+                "couldn't download URL %s to %s: %s" % (url, filename, e)
+            )
 
-        used_name = the_path.split('/')[-1]
+        used_name = the_path.split("/")[-1]
 
         # Trying to get better name from response headers
-        content_disposition = info.get('content-disposition')
+        content_disposition = info.get("content-disposition")
         if content_disposition:
             _, params = cgi.parse_header(content_disposition)
-            guessed_filename = params.get('filename') or used_name
+            guessed_filename = params.get("filename") or used_name
         else:
             guessed_filename = used_name
 
         # Falling back to content type guessing
         ext = self.splitext(guessed_filename)[1]
-        content_type = info.get('content-type')
+        content_type = info.get("content-type")
         if not ext and content_type:
             ext = mimetypes.guess_extension(content_type)
             if ext:
@@ -300,33 +345,34 @@ class TemplateCommand(BaseCommand):
         Like os.path.splitext, but takes off .tar, too
         """
         base, ext = posixpath.splitext(the_path)
-        if base.lower().endswith('.tar'):
+        if base.lower().endswith(".tar"):
             ext = base[-4:] + ext
             base = base[:-4]
         return base, ext
 
     def extract(self, filename):
         """
-        Extract the given file to a temporarily and return
+        Extract the given file to a temporary directory and return
         the path of the directory with the extracted content.
         """
-        prefix = 'django_%s_template_' % self.app_or_project
-        tempdir = tempfile.mkdtemp(prefix=prefix, suffix='_extract')
+        prefix = "django_%s_template_" % self.app_or_project
+        tempdir = tempfile.mkdtemp(prefix=prefix, suffix="_extract")
         self.paths_to_remove.append(tempdir)
         if self.verbosity >= 2:
-            self.stdout.write('Extracting %s' % filename)
+            self.stdout.write("Extracting %s" % filename)
         try:
             archive.extract(filename, tempdir)
             return tempdir
         except (archive.ArchiveException, OSError) as e:
-            raise CommandError("couldn't extract file %s to %s: %s" %
-                               (filename, tempdir, e))
+            raise CommandError(
+                "couldn't extract file %s to %s: %s" % (filename, tempdir, e)
+            )
 
     def is_url(self, template):
         """Return True if the name looks like a URL."""
-        if ':' not in template:
+        if ":" not in template:
             return False
-        scheme = template.split(':', 1)[0].lower()
+        scheme = template.split(":", 1)[0].lower()
         return scheme in self.url_schemes
 
     def make_writeable(self, filename):
