@@ -33,31 +33,34 @@ class EngineHandler:
             try:
                 # This will raise an exception if 'BACKEND' doesn't exist or
                 # isn't a string containing at least one dot.
-                default_name = tpl['BACKEND'].rsplit('.', 2)[-2]
+                default_name = tpl["BACKEND"].rsplit(".", 2)[-2]
             except Exception:
-                invalid_backend = tpl.get('BACKEND', '<not defined>')
+                invalid_backend = tpl.get("BACKEND", "<not defined>")
                 raise ImproperlyConfigured(
                     "Invalid BACKEND for a template engine: {}. Check "
-                    "your TEMPLATES setting.".format(invalid_backend))
+                    "your TEMPLATES setting.".format(invalid_backend)
+                )
 
             tpl = {
-                'NAME': default_name,
-                'DIRS': [],
-                'APP_DIRS': False,
-                'OPTIONS': {},
+                "NAME": default_name,
+                "DIRS": [],
+                "APP_DIRS": False,
+                "OPTIONS": {},
                 **tpl,
             }
 
-            templates[tpl['NAME']] = tpl
-            backend_names.append(tpl['NAME'])
+            templates[tpl["NAME"]] = tpl
+            backend_names.append(tpl["NAME"])
 
         counts = Counter(backend_names)
         duplicates = [alias for alias, count in counts.most_common() if count > 1]
         if duplicates:
             raise ImproperlyConfigured(
                 "Template engine aliases aren't unique, duplicates: {}. "
-                "Set a unique NAME for each engine in settings.TEMPLATES."
-                .format(", ".join(duplicates)))
+                "Set a unique NAME for each engine in settings.TEMPLATES.".format(
+                    ", ".join(duplicates)
+                )
+            )
 
         return templates
 
@@ -70,13 +73,14 @@ class EngineHandler:
             except KeyError:
                 raise InvalidTemplateEngineError(
                     "Could not find config for '{}' "
-                    "in settings.TEMPLATES".format(alias))
+                    "in settings.TEMPLATES".format(alias)
+                )
 
             # If importing or initializing the backend raises an exception,
             # self._engines[alias] isn't set and this code may get executed
             # again, so we must preserve the original params. See #24265.
             params = params.copy()
-            backend = params.pop('BACKEND')
+            backend = params.pop("BACKEND")
             engine_cls = import_string(backend)
             engine = engine_cls(params)
 
@@ -99,7 +103,7 @@ def get_app_template_dirs(dirname):
     installed applications.
     """
     template_dirs = [
-        str(Path(app_config.path) / dirname)
+        Path(app_config.path) / dirname
         for app_config in apps.get_app_configs()
         if app_config.path and (Path(app_config.path) / dirname).is_dir()
     ]
